@@ -1,3 +1,8 @@
+# Copyright (c) Facebook, Inc. and its affiliates.
+
+# This source code is licensed under the MIT license found in the
+# LICENSE file in the root directory of this source tree.
+
 library(prophet)
 context("Prophet tests")
 
@@ -28,8 +33,13 @@ test_that("fit_predict_no_changepoints", {
   expect_warning({
     # warning from prophet(), error from predict()
     m <- prophet(train, n.changepoints = 0)
-    expect_error(predict(m, future), NA)
   })
+  fcst <- predict(m, future)
+
+  expect_warning({
+    m <- prophet(train, n.changepoints = 0, mcmc.samples = 100)
+  })
+  fcst <- predict(m, future)
 })
 
 test_that("fit_predict_changepoint_not_in_history", {
@@ -68,6 +78,16 @@ test_that("fit_predict_constant_history", {
   expect_equal(tail(fcst$yhat, 1), 0)
 })
 
+test_that("fit_predict_uncertainty_disabled", {
+  skip_if_not(Sys.getenv('R_ARCH') != '/i386')
+  for (uncertainty in c(0, FALSE)) {
+    m <- prophet(train, uncertainty.samples = uncertainty)
+    fcst <- predict(m, future)
+    expected.cols <- c('ds', 'trend', 'additive_terms', 'weekly', 'multiplicative_terms', 'yhat')
+    expect_equal(expected.cols, colnames(fcst))
+  }
+})
+
 test_that("setup_dataframe", {
   history <- train
   m <- prophet(history, fit = FALSE)
@@ -83,6 +103,18 @@ test_that("setup_dataframe", {
   expect_equal(max(history$y_scaled), 1)
 })
 
+test_that("setup_names_errors", {
+  m <- prophet()
+  expect_error(
+    m <- add_seasonality(m, "3monthly"),
+    "You have provided a name that is not syntactically valid in R, 3monthly"
+  )
+  expect_error(
+    m <- add_regressor(m, "2monthsale"),
+    "You have provided a name that is not syntactically valid in R, 2monthsale"
+  )
+})
+
 test_that("logistic_floor", {
   skip_if_not(Sys.getenv('R_ARCH') != '/i386')
   skip_on_os('mac')  # Resolves mysterious CRAN build issue
@@ -97,6 +129,7 @@ test_that("logistic_floor", {
   expect_true(m$logistic.floor)
   expect_true('floor' %in% colnames(m$history))
   expect_equal(m$history$y_scaled[1], 1., tolerance = 1e-6)
+  expect_equal(m$fit.kwargs, list(algorithm = 'Newton'))
   fcst1 <- predict(m, future1)
 
   m2 <- prophet(growth = 'logistic')
@@ -516,6 +549,10 @@ test_that("custom_seasonality", {
                          holiday = c('special_day'),
                          prior_scale = c(4))
   m <- prophet(holidays=holidays)
+  expect_error(
+    add_seasonality(m, name="incorrect.fourier.order", period=30, fourier.order=-10),
+    "Fourier order must be > 0."
+  )
   m <- add_seasonality(m, name='monthly', period=30, fourier.order=5)
   true <- list(
     period = 30, fourier.order = 5, prior.scale = 10, mode = 'additive',
@@ -524,10 +561,12 @@ test_that("custom_seasonality", {
     expect_equal(m$seasonalities$monthly[[name]], true[[name]])
   }
   expect_error(
-    add_seasonality(m, name='special_day', period=30, fourier_order=5)
+    add_seasonality(m, name='special_day', period=30, fourier.order=5),
+    "already used for a holiday."
   )
   expect_error(
-    add_seasonality(m, name='trend', period=30, fourier_order=5)
+    add_seasonality(m, name='trend', period=30, fourier.order=5),
+    "is reserved."
   )
   m <- add_seasonality(m, name='weekly', period=30, fourier.order=5)
   # Test priors
